# Delete previous days for same time range before inserting data. Prevents duplicates from being inserted
INCREMENTAL_DELETE = """
    DELETE FROM drops_campaigns_overview WHERE record_date >= %(start_date)s AND record_date < %(end_date)s
"""

# Query Redshift and generate daily metrics in the time range
INCREMENTAL = """
WITH
drops_claimed as (
SELECT *
FROM tahoe.drops_claimed
WHERE "date" >= DATEADD(day, -31, %(start_date)s) AND "date" < DATEADD(day, 1, %(start_date)s)
)

, drops_mw_goal_complete AS (
SELECT *
FROM tahoe.drops_mw_goal_complete
WHERE "date" >= DATEADD(day, -31, %(start_date)s) AND "date" < DATEADD(day, 1, %(start_date)s)
)

, drops_server_campaigns AS (
SELECT campaignid AS campaign_id, gameid AS game_id, name,
TIMESTAMP 'epoch' + startdateunix * INTERVAL '1 second' as start_date,
TIMESTAMP 'epoch' + enddateunix * INTERVAL '1 second' as end_date
FROM dbsnapshots.drops2_campaigns
)

, drops_server_drops AS (
SELECT campaignid AS campaign_id, dropid AS drop_id, name,
CASE droptype
    WHEN 'TIME_BASED' THEN 'mw_event'
    WHEN 'EVENT_BASED' THEN 'event_based_event'
END AS drop_type
FROM dbsnapshots.drops2_drops
)

, drops_campaigns_metadata as (
SELECT a.campaign_id, game_id, a.name as campaign_name, drop_id, b.name as drop_name, drop_type
FROM drops_server_campaigns a
LEFT JOIN drops_server_drops b
    ON a.campaign_id = b.campaign_id
)

, drops_client_claimable_impression AS (
SELECT a.*, b.drop_type, b.game_id
FROM tahoe.drops_client_claimable_impression a
LEFT JOIN drops_campaigns_metadata b
    ON a.campaign_id = b.campaign_id
    AND a.drop_id = b.drop_id
WHERE "date" >= DATEADD(day, -31, %(start_date)s) AND "date" < DATEADD(day, 1, %(start_date)s)
)

, drops_community_callout_actions as (
SELECT time_utc, campaign_id, user_id, channel_id, location, action,
    CASE drop_type
        WHEN 'time_based' THEN 'mw_event'
        WHEN 'event_based' THEN 'event_based_event'
    END AS
    drop_type
FROM tahoe.drops_community_callout_actions a
WHERE "date" >= DATEADD(day, -31, %(start_date)s) AND "date" < DATEADD(day, 1, %(start_date)s)
AND drop_type != ''
)

, callout_with_games AS (
SELECT a.time_utc, a.channel_id, a.user_id, a.campaign_id, b.name as campaign_name,
a.drop_type, a.location, a.action, b.game_id
FROM drops_community_callout_actions a
LEFT JOIN drops_server_campaigns b
    on a.campaign_id = b.campaign_id
WHERE game_id != 0
AND %(start_date)s >= DATE_TRUNC('day', b.start_date) and %(start_date)s < b.end_date
GROUP BY 1,2,3,4,5,6,7,8,9
)

, viewers_eligible AS (
SELECT campaign_id, campaign_name, game_id,
count(distinct user_id) AS viewers_eligible
FROM callout_with_games
WHERE time_utc >= %(start_date)s  AND time_utc < DATEADD(day, 1, %(start_date)s)
    AND location = 'drops_info'
    AND action = 'render'
GROUP BY 1,2,3
)

, viewers_eligible_viewers AS (
SELECT campaign_id, game_id, drop_type, user_id
FROM callout_with_games
WHERE time_utc >= %(start_date)s  AND time_utc < DATEADD(day, 1, %(start_date)s)
    AND location = 'drops_info'
    AND action = 'render'
GROUP BY 1,2,3,4
)

, viewers_eligible_cumulative as (
SELECT a.campaign_id, a.game_id,
count(distinct a.user_id) as viewers_eligible_cumulative
FROM callout_with_games a
LEFT JOIN drops_server_campaigns b
    ON a.campaign_id = b.campaign_id
WHERE a.time_utc >= DATE_TRUNC('day', b.start_date) AND a.time_utc < DATEADD('day', 1, %(start_date)s)
    AND location = 'drops_info'
    AND action = 'render'
GROUP BY 1,2
)

, viewers_eligible_cumulative_viewers as (
SELECT a.campaign_id, a.game_id, a.drop_type, a.user_id
FROM callout_with_games a
LEFT JOIN drops_server_campaigns b
    ON a.campaign_id = b.campaign_id
WHERE a.time_utc >= DATE_TRUNC('day', b.start_date) AND a.time_utc < DATEADD('day', 1, %(start_date)s)
    AND location = 'drops_info'
    AND action = 'render'
)

, streamers_eligible AS (
SELECT campaign_id, game_id,
count(distinct channel_id) AS streamers_eligible
FROM callout_with_games
WHERE time_utc >= %(start_date)s  AND time_utc < DATEADD(day, 1, %(start_date)s)
    AND location = 'drops_info'
    AND action = 'render'
GROUP BY 1,2
)

, streamers_eligible_cumulative as (
SELECT a.campaign_id, a.game_id,
count(distinct a.channel_id) as streamers_eligible_cumulative
FROM callout_with_games a
LEFT JOIN drops_server_campaigns b
    ON a.campaign_id = b.campaign_id
WHERE a.time_utc >= DATE_TRUNC('day', b.start_date) AND a.time_utc < DATEADD('day', 1, %(start_date)s)
    AND location = 'drops_info'
    AND action = 'render'
GROUP BY 1,2
)

, event_drops_earned AS (
SELECT a.campaign_id, a.game_id,
count(distinct a.user_id) AS event_drops_earned
FROM drops_client_claimable_impression a
JOIN viewers_eligible_viewers b
    ON a.campaign_id = b.campaign_id
    AND a.game_id = b.game_id
    AND a.drop_type = b.drop_type
    AND a.user_id = b.user_id
WHERE a.time_utc >= %(start_date)s AND a.time_utc < DATEADD(day, 1, %(start_date)s)
AND a.drop_type = 'event_based_event'
GROUP BY 1,2
)

, event_drops_earned_cumulative as (
SELECT a.campaign_id, a.game_id,
count(distinct a.user_id) as event_drops_earned_cumulative
FROM drops_client_claimable_impression a
LEFT JOIN drops_server_campaigns b
    ON a.campaign_id = b.campaign_id
JOIN viewers_eligible_cumulative_viewers c
    ON a.campaign_id = c.campaign_id
    AND a.game_id = c.game_id
    AND a.drop_type = c.drop_type
    AND a.user_id = c.user_id
WHERE a.time_utc >= DATE_TRUNC('day', b.start_date) AND a.time_utc < DATEADD('day', 1, %(start_date)s)
    AND a.drop_type = 'event_based_event'
GROUP BY 1,2
)

, mw_drops_earned AS (
SELECT a.campaign_id, a.game_id,
count(distinct a.user_id) AS mw_drops_earned
FROM drops_mw_goal_complete a
JOIN viewers_eligible_viewers b
    ON a.campaign_id = b.campaign_id
    AND a.game_id = b.game_id
    AND a.drop_type = b.drop_type
    AND a.user_id = b.user_id
WHERE a.time_utc >= %(start_date)s  AND a.time_utc < DATEADD(day, 1, %(start_date)s)
GROUP BY 1,2
)

, mw_drops_earned_cumulative as (
SELECT a.campaign_id, a.game_id,
count(distinct a.user_id) AS mw_drops_earned_cumulative
FROM drops_mw_goal_complete a
LEFT JOIN drops_server_campaigns b
    ON a.campaign_id = b.campaign_id
JOIN viewers_eligible_cumulative_viewers c
    ON a.campaign_id = c.campaign_id
    AND a.game_id = c.game_id
    AND a.drop_type = c.drop_type
    AND a.user_id = c.user_id
WHERE a.time_utc >= DATE_TRUNC('day', b.start_date) AND a.time_utc < DATEADD('day', 1, %(start_date)s)
GROUP BY 1,2
)

, viewers_claimed as (
SELECT campaign_id, game_id,
count(distinct user_id) AS viewers_claimed
FROM drops_claimed
WHERE time_utc >= %(start_date)s AND time_utc < DATEADD(day, 1, %(start_date)s)
GROUP BY 1,2
)

, viewers_claimed_cumulative as (
SELECT a.campaign_id, a.game_id,
count(distinct user_id) as viewers_claimed_cumulative
FROM drops_claimed a
LEFT JOIN drops_server_campaigns b
    ON a.campaign_id = b.campaign_id
WHERE a.time_utc >= DATE_TRUNC('day', b.start_date) AND a.time_utc < DATEADD('day', 1, %(start_date)s)
GROUP BY 1,2
)

, viewers_linked_and_claimed as (
SELECT campaign_id, game_id,
count(distinct user_id) AS viewers_linked_and_claimed
FROM drops_claimed
WHERE time_utc >= %(start_date)s AND time_utc < DATEADD(day, 1, %(start_date)s)
    AND account_linked = true
GROUP BY 1,2
)

, viewers_linked_and_claimed_cumulative as (
SELECT a.campaign_id, a.game_id,
count(distinct user_id) as viewers_linked_and_claimed_cumulative
FROM drops_claimed a
LEFT JOIN drops_server_campaigns b
    ON a.campaign_id = b.campaign_id
WHERE a.time_utc >= DATE_TRUNC('day', b.start_date) AND a.time_utc < DATEADD('day', 1, %(start_date)s)
    AND account_linked = true
GROUP BY 1,2
)

SELECT
%(start_date)s::DATE AS record_date,
a.campaign_id,
a.game_id,
metadata.name as game_name,
coalesce(streamers_eligible, 0) as streamers_eligible,
coalesce(viewers_eligible, 0) as viewers_eligible,
coalesce(event_drops_earned, 0) as event_drops_earned,
coalesce(mw_drops_earned, 0) as mw_drops_earned,
coalesce(viewers_claimed, 0) as viewers_claimed,
coalesce(viewers_linked_and_claimed, 0) as viewers_linked_and_claimed,
coalesce(streamers_eligible_cumulative, 0) as streamers_eligible_accum,
coalesce(viewers_eligible_cumulative, 0) as viewers_eligible_accum,
coalesce(event_drops_earned_cumulative, 0) as event_drops_earned_accum,
coalesce(mw_drops_earned_cumulative, 0) as mw_drops_earned_accum,
coalesce(viewers_claimed_cumulative, 0) as viewers_claimed_accum,
coalesce(viewers_linked_and_claimed_cumulative, 0) as viewers_linked_and_claimed_accum,
a.campaign_name
FROM viewers_eligible a
LEFT JOIN event_drops_earned b
    ON a.campaign_id = b.campaign_id
    AND a.game_id = b.game_id
LEFT JOIN mw_drops_earned c
    ON a.campaign_id = c.campaign_id
    AND a.game_id = c.game_id
LEFT JOIN viewers_claimed d
    ON a.campaign_id = d.campaign_id
    AND a.game_id = d.game_id
LEFT JOIN viewers_linked_and_claimed e
    ON a.campaign_id = e.campaign_id
    AND a.game_id = e.game_id
LEFT JOIN streamers_eligible f
    ON a.campaign_id = f.campaign_id
    AND a.game_id = f.game_id
LEFT JOIN streamers_eligible_cumulative g
    ON a.campaign_id = g.campaign_id
    AND a.game_id = g.game_id
LEFT JOIN viewers_eligible_cumulative h
    ON a.campaign_id = h.campaign_id
    AND a.game_id = h.game_id
LEFT JOIN event_drops_earned_cumulative i
    ON a.campaign_id = i.campaign_id
    AND a.game_id = i.game_id
LEFT JOIN mw_drops_earned_cumulative j
    ON a.campaign_id = j.campaign_id
    AND a.game_id = j.game_id
LEFT JOIN viewers_claimed_cumulative k
    ON a.campaign_id = k.campaign_id
    AND a.game_id = k.game_id
LEFT JOIN viewers_linked_and_claimed_cumulative l
    ON a.campaign_id = l.campaign_id
    AND a.game_id = l.game_id
LEFT JOIN dbsnapshots.categories metadata
    ON a.game_id = metadata.id
"""


UPDATE_DOMAIN_REPORT_META_DATA = """
BEGIN;
DELETE FROM latest_domain_report_meta_data WHERE domain = 'drops' AND report = 'campaigns_overview';
INSERT INTO latest_domain_report_meta_data
SELECT %(end_date)s
    , 'drops'
    , 'campaigns_overview'
    , campaign_id
    , campaign_name
    , MIN(record_date)
    , MAX(record_date)
FROM drops_campaigns_overview
WHERE record_date >= '2020-03-20' AND record_date < %(end_date)s
GROUP BY campaign_id, campaign_name;
COMMIT;
"""
